<?php

namespace Drupal\mautic_paragraph\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\mautic_paragraph\MauticParagraphApi;
use Drupal\mautic_paragraph\MauticParagraphService;
use Mautic\Exception\RequiredParameterMissingException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

/**
 * Provides settings form to configure Mautic Paragraph integration settings.
 */
class MauticSettingsForm extends ConfigFormBase {
  use StringTranslationTrait;

  /**
   * The MauticService.
   *
   * @var \Drupal\mautic_paragraph\MauticParagraphService
   */
  protected $mauticParagraphService;

  /**
   * The Mautic config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $mauticConfig;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The MauticApi.
   *
   * @var \Drupal\mautic_paragraph\MauticParagraphApi
   */
  protected $mauticParagraphApi;

  /**
   * MauticSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\mautic_paragraph\MauticParagraphService $mauticParagraphService
   *   The Mauticservice.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\mautic_paragraph\MauticParagraphApi $mauticParagraphApi
   *   The MauticApi service.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              MauticParagraphService $mauticParagraphService,
                              MessengerInterface $messenger,
                              MauticParagraphApi $mauticParagraphApi) {
    parent::__construct($config_factory);
    $this->mauticConfig = $config_factory->getEditable('mautic_paragraph.settings');
    $this->mauticParagraphService = $mauticParagraphService;
    $this->messenger = $messenger;
    $this->mauticParagraphApi = $mauticParagraphApi;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('mautic_paragraph_service'),
      $container->get('messenger'),
      $container->get('mautic_paragraph_api')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['mautic_paragraph.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mautic_paragraph_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $form['auth_method'] = [
      '#type' => 'radios',
      '#title' => $this->t('Authentication method'),
      '#description' => $this->t('Choose an authentication method to use for this Mautic server.'),
      '#empty_value' => '',
      '#options' => [
        'basic_auth' => 'Basic Authentication',
        'oauth' => 'OAuth2',
      ],
      '#default_value' => $this->mauticConfig->get('auth_method'),
      '#required' => TRUE,
    ];

    $form['baseurl'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base url:'),
      '#required' => TRUE,
      '#default_value' => $this->mauticConfig->get('baseurl'),
      '#description' => 'The base url of your Mautic instance can be retrieved under general settings. Example baseurl: mautic.example.com',
    ];

    $form['basicauth'] = [
      '#type' => 'details',
      '#title' => t('Mautic Integration BasicAuth settings'),
      '#open' => TRUE,
      '#description' => $this->t(
        'Get your username and password from the Mautic instance.'
      ),
      '#states' => [
        'visible' => [
          ':input[name="auth_method"]' => ['value' => 'basic_auth'],
        ],
      ],
    ];

    $form['basicauth']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => $this->mauticConfig->get('username'),
    ];

    $form['basicauth']['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#description' => $this->t('If this field is left blank and the HTTP username is filled out, the current password will not be changed.'),
    ];

    $form['oauth'] = [
      '#type' => 'details',
      '#title' => t('Mautic Integration OAuth settings'),
      '#open' => TRUE,
      '#description' => $this->t(
        'To retrieve your client ID & secret: <ol>
          <li>Visit your mautic instance and retrieve your base url under general settings</li>
          <li>Create API credentials for OAuth under  API Credentials</li>
          <li>Retrieve your client ID/Secret when clicking edit on the selected API credentials item.</li>
          </ol>'
      ),
      '#states' => [
        'visible' => [
          ':input[name="auth_method"]' => ['value' => 'oauth'],
        ],
      ],
    ];

    $form['oauth']['clientid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID:'),
      '#description' => $this->t('The mautic integration client ID.'),
      '#default_value' => $this->mauticConfig->get('clientid'),
    ];

    $form['oauth']['clientsecret'] = [
      '#type' => 'password',
      '#title' => $this->t('Client Secret:'),
      '#description' => $this->t('If this field is left blank and the clientid is filled out, the current clientsecret will not be changed..'),
    ];

    $form['actions']['submit']['#value'] = $this->t('Test mautic connection / Save settings');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $values = $form_state->getValues();

    switch ($values['auth_method']) {

      case 'basic_auth':
        if (empty($values['username'])) {
          $form_state->setErrorByName('username', $this->t('The username cannot be empty. Fill in the correct username.'));
        }
        break;

      case 'oauth':
        if (empty($values['clientid'])) {
          $form_state->setErrorByName('clientid', $this->t('The clientid cannot be empty. Fill in the correct clientid.'));
        }
        break;
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    switch ($values['auth_method']) {
      case 'basic_auth':
        $username = $this->mauticConfig->get('username');
        $password = $this->mauticConfig->get('password');

        // For password fields, there is no default value, they're empty
        // by default.
        // Therefore we ignore empty submissions if the user
        // didn't change either.
        if ($values['password'] === ''
          && isset($username)
          && $values['username'] === $username) {
          $values['password'] = $password;
        }
        try {
          $this->mauticParagraphService->verifyCredentials($values['username'], $values['password'], $values['baseurl']);

          // If no error message, we can save the form.
          $this->mauticConfig
            ->set('baseurl', $values['baseurl'])
            ->set('username', $values['username'])
            ->set('password', $values['password'])
            ->set('auth_method', $values['auth_method'])
            ->save();
          $this->messenger()->addStatus($this->t('Connection established via Basic Auth! The configuration options have been saved.'));
        }
        catch (UnauthorizedHttpException $e) {
          watchdog_exception('error', $e);
          $this->messenger->addError($this->t('Failed to retrieve forms. Your credentials in the Mautic Integration settings page are wrong.'));
        }
        catch (RequiredParameterMissingException $e) {
          watchdog_exception('error', $e);
          $this->messenger->addError($this->t('Connection failed, missing credentials.'));
        }
        catch (\Exception $e) {
          $this->messenger->addError('Connection failed, there was a problem with the connection to the Mautic API instance.');
          watchdog_exception('error', $e);
        }
        break;

      case 'oauth':
        $clientid = $this->mauticConfig->get('clientid');
        $clientsecret = $this->mauticConfig->get('clientsecret');

        // For password fields, there is no default value, they're empty
        // by default.
        // Therefore we ignore empty submissions if the user
        // didn't change either.
        if ($values['clientsecret'] === ''
          && isset($clientid)
          && $values['clientid'] === $clientid) {
          $values['clientsecret'] = $clientsecret;
        }

        $this->mauticConfig
          ->set('baseurl', $values['baseurl'])
          ->set('clientid', $values['clientid'])
          ->set('clientsecret', $values['clientsecret'])
          ->set('auth_method', $values['auth_method'])
          ->save();
        // Start authorization flow OAuth2.
        $this->mauticParagraphApi->startAuthorization();
        break;
    }
  }

}
