<?php

namespace Drupal\mautic_paragraph;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Mautic\Auth\ApiAuth;
use Mautic\Exception\RequiredParameterMissingException;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Create basic auth or OAuth connection with Mautic.
 */
class MauticParagraphApi {

  use StringTranslationTrait;

  /**
   * The Mautic config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $mauticConfig;

  /**
   * Drupal state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * The current request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a MauticApi class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $url_generator
   *   The URL Generator service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, UrlGeneratorInterface $url_generator, RequestStack $request_stack, MessengerInterface $messenger) {
    $this->mauticConfig = $config_factory->get('mautic_paragraph.settings');
    $this->state = $state;
    $this->urlGenerator = $url_generator;
    $this->request = $request_stack->getCurrentRequest();
    $this->messenger = $messenger;
  }

  /**
   * Starts the OAuth authorization flow.
   */
  public function startAuthorization() {
    if (!empty($this->mauticConfig->get('clientid')) && !empty($this->mauticConfig->get('clientsecret'))
        && !empty($this->mauticConfig->get('baseurl'))) {
      // Clear the old access token data.
      $session = $this->request->getSession();
      $session->remove('oauth2state');

      $this->state->set('mautic_access_token_data', '');

      $this->createMauticProvider();
    }
    else {
      $this->messenger->addError('Missing credentials.');
    }
  }

  /**
   * Create Mautic Provider for OAuth.
   *
   * @param bool $redirect
   *   Redirect for authorization.
   *
   * @return \Mautic\Auth\AuthInterface
   *   Return Auth object from mautic.
   */
  public function createMauticProvider($redirect = TRUE) {
    $settings = [
      'baseUrl'          => 'https://' . $this->mauticConfig->get('baseurl'),
      'version'          => 'OAuth2',
      'clientKey'        => $this->mauticConfig->get('clientid'),
      'clientSecret'     => $this->mauticConfig->get('clientsecret'),
      'callback'         => Url::fromRoute('mautic_paragraph.oauth_callback')->setAbsolute()->toString(),
    ];

    $oauth = $this->state->get('mautic_access_token_data');

    if ($oauth) {
      $settings += [
        'accessToken' => $oauth['access_token'],
        'accessTokenExpires' => $oauth['expires'],
        'refreshToken' => $oauth['refresh_token'],
      ];
    }

    // Initiate the auth object.
    $initAuth = new ApiAuth();
    $auth = $initAuth->newAuth($settings);

    try {
      if ($auth->validateAccessToken($redirect)) {
        if ($auth->accessTokenUpdated()) {
          $accessTokenData = $auth->getAccessTokenData();

          $this->state->set('mautic_access_token_data', $accessTokenData);
        }
      }
    }
    catch (Exception $e) {
      $this->messenger->addError($e->getMessage());
      watchdog_exception('error', $e);
    }
    return $auth;
  }

  /**
   * Get API client.
   *
   * @param string $username
   *   Username.
   * @param string $password
   *   Password.
   *
   * @return \Mautic\Auth\AuthInterface
   *   Returns BasicAuth object.
   *
   * @throws \Mautic\Exception\RequiredParameterMissingException
   */
  public function getApiClient($username = NULL, $password = NULL) {
    $auth = new ApiAuth();

    if (empty($username) && empty($password)) {
      $username = $this->mauticConfig->get('username');
      $password = $this->mauticConfig->get('password');
    }
    if ($username && $password) {
      $settings = [
        'userName' => $username,
        'password' => $password,
      ];
      return $auth->newAuth($settings, 'BasicAuth');
    }
    else {
      throw new RequiredParameterMissingException();
    }
  }

}
