<?php

namespace Drupal\mautic_paragraph;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Exception;
use Mautic\Exception\ContextNotFoundException;
use Mautic\Exception\RequiredParameterMissingException;
use Mautic\MauticApi;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

/**
 * Provides functions for integration with Mautic.
 */
class MauticParagraphService {

  use StringTranslationTrait;
  /**
   * The mautic API service.
   *
   * @var \Drupal\mautic_paragraph\MauticParagraphApi
   */
  protected $mauticApi;

  /**
   * The mautic config settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $mauticConfig;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The cache.default cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Constructs MauticService class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\mautic_paragraph\MauticParagraphApi $mauticApi
   *   The MauticApi.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger provider.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   The cache object associated with the default bin.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MauticParagraphApi $mauticApi, MessengerInterface $messenger, CacheBackendInterface $cache_backend) {
    $this->mauticApi = $mauticApi;
    $this->mauticConfig = $config_factory->get('mautic_paragraph.settings');
    $this->messenger = $messenger;
    $this->cacheBackend = $cache_backend;
  }

  /**
   * Provides a list of forms.
   *
   * @param string $input
   *   Search string for fetching forms.
   *
   * @return array
   *   List of forms from Mautic.
   */
  public function getList($input = NULL) {
    // Get cached data.
    $cache = $this->cacheBackend->get('mautic_form_list');
    $data = $cache->data ?? [];

    // If no cached data, fetch data from API.
    if (!$data) {
      try {
        // Retrieve Auth implementation for right auth method.
        $auth = $this->getAuth();

        // Get baseurl from config.
        $baseurl = $this->mauticConfig->get('baseurl');

        // Make new mauticAPI instance.
        $api = new MauticApi();

        // Get formsAPI.
        $formsApi = $api->newApi('forms', $auth, 'http://' . $baseurl);

        // Cache the full list when it's not currently cached.
        $response = $formsApi->getList('', 0, 100, '', 'ASC', TRUE, TRUE);
        // Check for errors.
        if (isset($response['errors'])) {
          if ($response['errors']) {
            // Get error message.
            $error_message = $this->getErrorMessage($response);

            // Check if unauthorized error code, throws another exception.
            if ($response['errors']['0']['code'] == 401) {
              throw new UnauthorizedHttpException(' Basic realm="Mautic integration module', $error_message);
            }
            throw new \Exception($error_message);
          }
        }
        $this->cacheBackend->set('mautic_form_list', $response['forms'], time() + 3600, [
          'config:mautic_paragraph.settings',
        ]);
        $data = $response['forms'];
      }
      catch (RequiredParameterMissingException $e) {
        watchdog_exception('error', $e);
        // Display credentials error message.
        $this->getCredentialsErrorMessage();
      }
      catch (ContextNotFoundException $e) {
        watchdog_exception('error', $e);
        $this->messenger->addError($this->t('Failed to retrieve forms, invalid Context.'));
      }
      catch (UnauthorizedHttpException $e) {
        watchdog_exception('error', $e);
        $this->messenger->addError($this->t('Failed to retrieve forms. Your credentials in the Mautic Integration settings page are wrong.'));
      }
      catch (\Exception $e) {
        watchdog_exception('error', $e);
        $this->messenger->addError($this->t('Failed to retrieve forms. Verify your settings on the Mautic paragraph Integration settings page.'));
      }
    }
    if ($input) {
      $data = array_filter($data, function ($k) use (&$input) {
        return strpos(strtolower($k['name']), strtolower($input)) !== FALSE;
      });
      return $data;
    }
    return $data;
  }

  /**
   * Fetches the form title from an id.
   *
   * @param int $id
   *   The form id.
   *
   * @return string
   *   Name of the form.
   */
  public function getFormTitle($id) {
    // Get unfiltered list of forms.
    $formList = $this->getList();

    // Loop over formList to get name of matching form.
    $form = array_filter($formList, function ($item) use (&$id) {
      return $item['id'] == $id;
    });

    // When there is no match, return null.
    if (count($form) === 0) {
      return NULL;
    }
    return array_shift($form)['name'];
  }

  /**
   * Throws error when credentials are wrong.
   *
   * @param string $username
   *   The username.
   * @param string $password
   *   The password.
   * @param string $baseurl
   *   The baseurl for Mautic connection.
   *
   * @throws \Mautic\Exception\ContextNotFoundException
   * @throws \Mautic\Exception\RequiredParameterMissingException
   * @throws \Exception
   */
  public function verifyCredentials($username, $password, $baseurl) {

    // Retrieve ApiClient from MauticApi service.
    $auth = $this->mauticApi->getApiClient($username, $password);

    // Make new mauticAPI instance.
    $api = new MauticApi();

    // Get formsAPI.
    $formsApi = $api->newApi('users', $auth, 'http://' . $baseurl);

    // Get list.
    $response = $formsApi->getList('', 0, 0, '', 'ASC', TRUE, TRUE);

    // If error message, throw exception.
    if (isset($response['errors'])) {
      $error_message = $this->getErrorMessage($response);
      if ($response['errors']['0']['code'] == 401) {
        throw new UnauthorizedHttpException(' Basic realm="Mautic integration module', $error_message);
      }
      throw new \Exception($error_message);
    }
  }

  /**
   * Returns a response message based on Mautic API response.
   *
   * @param array|mixed $response
   *   Mautic API response array.
   *
   * @return string
   *   A proper response message.
   */
  protected function getErrorMessage($response) {
    return explode('Response:', $response['errors']['0']['message'])[0];
  }

  /**
   * Display credentials error message.
   */
  protected function getCredentialsErrorMessage() {
    // Get link from route settings page.
    $link = Url::fromRoute('mautic_paragraph.route_settings');
    // Make the link.
    $link_message = Link::fromTextAndUrl($this->t('mautic settings page'), $link)->toString();
    // Display error.
    $this->messenger->addError($this->t('Failed to retrieve forms, missing credentials. Check %link .', ['%link' => $link_message]));
  }

  /**
   * Returns the right auth implementation.
   *
   * @return null|\Mautic\Auth\AuthInterface
   *   Returns auth implementation or null if none.
   *
   * @throws \Mautic\Exception\RequiredParameterMissingException
   */
  protected function getAuth() {
    // Get selected authentication method.
    $auth_method = $this->mauticConfig->get('auth_method');

    $auth = NULL;
    switch ($auth_method) {
      case 'basic_auth':
        $auth = $this->mauticApi->getApiClient();
        break;

      case 'oauth':
        $auth = $this->mauticApi->createMauticProvider(FALSE);
        break;
    }
    return $auth;
  }

}
